use crate::{schema, UIDCError, key::KeyType};
use microrm::prelude::*;

pub fn create(realm: &schema::Realm, name: &String, key_type: KeyType) -> Result<(), UIDCError> {
    let rng = ring::rand::SystemRandom::new();
    let client_secret : [u8; 32] = ring::rand::generate(&rng).unwrap().expose();

    realm.clients.insert(schema::Client {
        shortname: name.into(),
        secret: base64::encode(&client_secret),
        key_type: key_type.into(),
        redirects: Default::default(),
        scopes: Default::default(),
    })?;
    Ok(())
}

pub fn inspect(realm: &schema::Realm, name: &String) -> Result<(), UIDCError> {
    if let Some(client) = realm.clients.unique(name).get()? {
        println!("Found client {name}");
        println!("Client secret: {}", client.secret);
        println!("Signature type: {:?}", client.key_type);
        println!("Valid redirect URIs:");
        for redirect in client.redirects.get()? {
            println!(" - {}", redirect.redirect);
        }

        println!("Valid scopes:");
        for scope in client.scopes.get()? {
            println!(" - {}", scope.shortname);
        }
    }
    else {
        println!("No such client {name}");
    }

    Ok(())
}
