// use crate::{key::KeyError, token::TokenError, user::UserError};
use crate::{key::KeyError,user::UserError};

#[derive(Debug)]
pub enum UIDCError {
    /// aborting operation because of invalid state
    Abort(&'static str),
    /// aborting operation because of invalid state
    AbortString(String),
    /// database access issue or violated constraint
    DatabaseError(microrm::Error),

    /// error with key generation or message signing
    KeyError(KeyError),

    /*
    /// error with token generation or verification
    TokenError(TokenError),
    */

    /// error with user operation
    UserError(UserError),
}

impl std::fmt::Display for UIDCError {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        <Self as std::fmt::Debug>::fmt(self, f)
    }
}

impl std::error::Error for UIDCError {}

impl From<microrm::Error> for UIDCError {
    fn from(value: microrm::Error) -> Self {
        Self::DatabaseError(value)
    }
}

macro_rules! error_converter {
    ($toconv:ident) => {
        impl From<$toconv> for UIDCError {
            fn from(value: $toconv) -> Self {
                Self::$toconv(value)
            }
        }
    };
}

error_converter!(KeyError);
error_converter!(UserError);
/*
error_converter!(TokenError);
*/
