use crate::{schema, UIDCError};
use microrm::prelude::*;

pub fn create_group(
    realm: &schema::Realm,
    name: &String,
) -> Result<(), UIDCError> {
    realm.groups.insert(schema::Group {
        shortname: name.into(),
        roles: Default::default(),
        users: Default::default(),
    })?;
    Ok(())
}

pub fn list_groups(
    realm: &schema::Realm,
) -> Result<(), UIDCError> {
    for group in realm.groups.get()? {
        println!("{}", group.shortname);
    }
    Ok(())
}

pub fn list_members(
    realm: &schema::Realm,
    name: &String,
) -> Result<(), UIDCError> {
    for member in realm.groups.unique(name).join(schema::Group::Users).get()? {
        println!("- {}", member.username);
    }

    Ok(())
}

pub fn list_roles(
    realm: &schema::Realm,
    name: &String,
) -> Result<(), UIDCError> {
    for role in realm.groups.unique(name).join(schema::Group::Roles).get()? {
        println!("- {}", role.shortname);
    }

    Ok(())
}

pub fn attach_user(
    realm: &schema::Realm,
    group_name: &String,
    username: &String,
) -> Result<(), UIDCError> {
    let group = realm.groups.unique(group_name).get()?;
    let user = realm.users.unique(username).get()?;

    match (group, user) {
        (None, _) => Err(UIDCError::Abort("no such group")),
        (_, None) => Err(UIDCError::Abort("no such user")),
        (Some(group), Some(user)) => {
            group.users.connect_to(user.id())?;
            Ok(())
        }
    }
}

pub fn detach_user(
    realm: &schema::Realm,
    group_name: &String,
    username: &String,
) -> Result<(), UIDCError> {
    let group = realm.groups.unique(group_name).get()?;
    let user = realm.users.unique(username).get()?;

    match (group, user) {
        (None, _) => Err(UIDCError::Abort("no such group")),
        (_, None) => Err(UIDCError::Abort("no such user")),
        (Some(group), Some(user)) => {
            group.users.disconnect_from(user.id())?;
            Ok(())
        }
    }
}

pub fn attach_role(
    realm: &schema::Realm,
    group_name: &String,
    role_name: &String
) -> Result<(), UIDCError> {
    let group = realm.groups.unique(group_name).get()?;
    let role = realm.roles.unique(role_name).get()?;

    match (group, role) {
        (None, _) => Err(UIDCError::Abort("no such group")),
        (_, None) => Err(UIDCError::Abort("no such role")),
        (Some(group), Some(role)) => {
            group.roles.connect_to(role.id())?;
            Ok(())
        }
    }
}

pub fn detach_role(
    realm: &schema::Realm,
    group_name: &String,
    role_name: &String
) -> Result<(), UIDCError> {
    let group = realm.groups.unique(group_name).get()?;
    let role = realm.roles.unique(role_name).get()?;

    match (group, role) {
        (None, _) => Err(UIDCError::Abort("no such group")),
        (_, None) => Err(UIDCError::Abort("no such role")),
        (Some(group), Some(role)) => {
            group.roles.disconnect_from(role.id())?;
            Ok(())
        }
    }
}
